import { __decorate } from "tslib";
import { Connect } from '@genesislcap/foundation-comms';
import { ZeroAgSelectRenderer, ZeroAgTextRenderer } from '@genesislcap/foundation-zero-grid-pro';
import { customElement, FASTElement, observable } from '@microsoft/fast-element';
import { filterObjectArray, getFiltersByFieldType } from '../../utils';
import { AVAILABLE_FIELDS_CONFIG, SELECTED_FILTERS_CONFIG } from './columns';
import { SelectFiltersStyles } from './select-filters.styles';
import { SelectFiltersTemplate } from './select-filters.template';
let SelectFilters = class SelectFilters extends FASTElement {
    constructor() {
        super(...arguments);
        this.filtersToRemove = [];
        this.selectedFields = [];
        this.selectedFilters = [];
        this.filterGridReady = false;
        // TODO: extract to util
        this.makeSureFieldValueIsNotLost = (array) => {
            return array.map((field) => {
                const matchingField = this.selectedFilters?.find((f) => f?.name === field?.name);
                return {
                    ...field,
                    ...matchingField,
                };
            });
        };
        this.onFilterSelect = (value, params) => {
            const newFields = this.selectedFilters.map((field) => {
                if (field.name === params.node.data.name) {
                    field.filter.conditional = value;
                }
                return field;
            });
            this.setSelectedFilters(newFields);
        };
        this.onFilterValueChange = (value, params) => {
            const newFields = this.selectedFilters.map((field) => {
                if (field.name === params.node.data.name) {
                    field.filter.value = value;
                }
                return field;
            });
            this.setSelectedFilters(newFields);
        };
        /**
         * Remove fields from selected filters grid and deselect them from the available fields grid
         */
        this.removeSelectedFilters = () => {
            // filter fields from selected list
            const filters = filterObjectArray('name', this.selectedFilters, this.filtersToRemove);
            this.setSelectedFilters(filters);
            // reset removal array
            this.filtersToRemove = [];
            // deselect available fieldgrid
            this.availableFieldsGrid.gridApi.deselectAll();
            // reselect
            this.availableFieldsGrid.gridApi.forEachNode((node) => {
                filters.find((field) => field.name === node.data.name) && node.setSelected(true);
            });
        };
    }
    connectedCallback() {
        super.connectedCallback();
        this.loadAvailableFieldsGrid();
        this.loadSelectedFiltersGrid();
        // TODO: this could be a util
        this.availableFieldsGrid.addEventListener('onGridReady', () => {
            window.addEventListener('resize', () => {
                setTimeout(() => {
                    this.availableFieldsGrid.gridApi.sizeColumnsToFit();
                });
            });
            this.availableFieldsGrid.gridApi.addEventListener('firstDataRendered', () => {
                this.availableFieldsGrid.gridApi.sizeColumnsToFit();
                this.selectDefaultFilters();
            });
            this.availableFieldsGrid.gridApi.addEventListener('selectionChanged', () => {
                this.setSelectedFilters([
                    ...this.makeSureFieldValueIsNotLost(this.availableFieldsGrid.gridApi.getSelectedRows()),
                ]);
            });
        });
        this.selectedFiltersGrid.addEventListener('onGridReady', () => {
            this.filterGridReady = true;
            window.addEventListener('resize', () => {
                setTimeout(() => {
                    this.selectedFiltersGrid.gridApi.sizeColumnsToFit();
                });
            });
            this.selectedFiltersGrid.gridApi.addEventListener('firstDataRendered', () => {
                this.selectedFiltersGrid.gridApi.sizeColumnsToFit();
            });
            this.selectedFiltersGrid.gridApi.addEventListener('selectionChanged', () => {
                this.filtersToRemove = this.selectedFiltersGrid.gridApi.getSelectedRows();
            });
        });
    }
    selectedFiltersChanged() {
        if (this.filterGridReady) {
            this.selectedFiltersGrid.gridApi.setRowData(this.selectedFilters);
            this.selectedFiltersGrid.gridApi.refreshCells();
        }
    }
    /**
     * Update selected filters on parent component
     */
    setSelectedFilters(filters) {
        this.$emit('setSelectedFilters', { filters });
    }
    /**
     * Get a simplified / normalized version of the filters
     * e.g. ['REPORT_ID', 'REPORT_NAME']
     * @returns Array of string
     */
    getNormalizedFilters() {
        return this.selectedFilters?.map(({ FIELD_NAME }) => FIELD_NAME);
    }
    /**
     * Loads availableFieldsGrid with data from the field selection step
     */
    async loadAvailableFieldsGrid() {
        this.availableFieldsGrid.gridOptions = {
            ...AVAILABLE_FIELDS_CONFIG.gridOptions,
            columnDefs: AVAILABLE_FIELDS_CONFIG.colDefs,
            rowData: [...this.selectedFields],
        };
    }
    /**
     * Loads the selectedFiltersGrid
     */
    loadSelectedFiltersGrid() {
        this.selectedFiltersGrid.gridOptions = {
            ...SELECTED_FILTERS_CONFIG.gridOptions,
            columnDefs: [
                ...SELECTED_FILTERS_CONFIG.colDefs,
                {
                    field: 'TYPE',
                    headerName: 'Filter',
                    maxWidth: 170,
                    cellStyle: {
                        overflow: 'visible',
                        paddingLeft: 0,
                        paddingRight: 0,
                    },
                    cellRenderer: ZeroAgSelectRenderer,
                    cellRendererParams: {
                        accessor: 'filter.conditional',
                        defaultValue: 'Select filter',
                        onSelect: this.onFilterSelect,
                        options: (data) => getFiltersByFieldType(data.type),
                    },
                },
                {
                    cellRenderer: ZeroAgTextRenderer,
                    cellRendererParams: {
                        onChange: this.onFilterValueChange,
                        accessor: 'filter.value',
                    },
                    field: 'filter.value',
                    headerName: 'Value',
                },
            ],
            rowData: [...this.filtersToRemove, ...this.selectedFilters],
        };
    }
    /**
     * Select default filters
     */
    selectDefaultFilters() {
        if (this.selectedFilters.length) {
            this.availableFieldsGrid.gridApi.forEachNode((node) => {
                this.selectedFilters.find((field) => field.name === node.data.name) &&
                    node.setSelected(true);
            });
            this.setSelectedFilters([
                ...this.makeSureFieldValueIsNotLost(this.availableFieldsGrid.gridApi.getSelectedRows()),
            ]);
        }
    }
};
__decorate([
    Connect
], SelectFilters.prototype, "connect", void 0);
__decorate([
    observable
], SelectFilters.prototype, "filtersToRemove", void 0);
__decorate([
    observable
], SelectFilters.prototype, "selectedFields", void 0);
__decorate([
    observable
], SelectFilters.prototype, "selectedFilters", void 0);
__decorate([
    observable
], SelectFilters.prototype, "filterGridReady", void 0);
SelectFilters = __decorate([
    customElement({
        name: 'reporting-select-filters',
        template: SelectFiltersTemplate,
        styles: SelectFiltersStyles,
    })
], SelectFilters);
export { SelectFilters };
